/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/* Created the junuary 1, 2008
 * Last change the junuary 1, 2008 */


#include <SDL/SDL.h>

#include "sokoban.h"
#include "i18n.h"
#include "object.h"


/**
 * \defgroup object Object management
 * Creation, management & delete of objects
 */

/** @{ */


/**
 * \brief Object structure
 * \struct _Object
 */


/**
 * \def OBJECT
 * \brief Define to cast to Object
 * \param v the variable to cast
 */

/**
 * \def object_getSurface
 * \brief Get SDL_Surface variable of an object
 * \param obj The object to trait
 * 
 * \see _Object
 */
/**
 * \def object_getPos
 * \brief Get SDL_Rect variable of an object
 * \param obj The object to trait
 * 
 * \see _Object
 */

/**
 * \brief Create a new object
 * \return A newly allocated object that should be freed with object_free()
 */
Object* object_new (void)
{
   trace ();
   
   Object *obj = malloc (sizeof *obj);
   
   if (obj == NULL)
      memalloc_errormsg ();
   else
      obj->surface = NULL;
   
   return obj;
}

/**
 * \brief Create a new object with some default values.
 * \param surface the SDL_Surface of the object
 * \param x the position in X of the object
 * \param y the positio nin Y of the object
 * \return a newly allocated object that should be free with object_free()
 * 
 * \see object_new()
 */
Object* object_newWithData (SDL_Surface *surface, int x, int y)
{
   trace ();
   
   Object *obj = object_new ();
   
   if (obj)
      object_setData (obj, surface, x, y);
   
   return obj;
}

/**
 * \brief Create a new object with some default values.
 * \param surface the SDL_Surface of the object
 * \param pos the SDL_Rect of the object
 * \return a newly allocated object that should be free with object_free()
 * 
 * \see object_new()
 */
Object* object_newWithDataSDL (SDL_Surface *surface, SDL_Rect pos)
{
   trace ();
   
   Object *obj = object_new ();
   
   if (obj)
      object_setDataFromSDL (obj, surface, pos);
   
   return obj;
}

/**
 * \brief Free an Object structure
 * \param obj the object to free
 */
void object_free (Object *obj)
{
   trace ();
   
   SDL_Surface *s;
   
   if (!obj) return;
   
   if ((s = object_getSurface (obj)))
      SDL_FreeSurface (s);
   
   free (obj);
}

/**
 * \brief Set the SDL_Surface of the object
 * \param obj the object to modify
 * \param surface the SDL_Surface to put in the object
 * \return \param obj
 */
Object* object_setSurface (Object *obj, SDL_Surface *surface)
{
   trace ();
   
   if (obj)
   {
      if (obj->surface != NULL)
         SDL_FreeSurface (obj->surface);
      
      obj->surface = surface;
   }
   
   if (!object_getSurface (obj))
      fprintf (stderr, _(" * SDL_error: %s\n"), SDL_GetError ());
   
   return obj;
}

/**
 * \brief Set the SDL_Rect of an object
 * \param obj the object to modify
 * \param pos the SDL_Rect to put in the object
 * \return \param obj
 */
Object* object_setPosFromRect (Object *obj, SDL_Rect pos)
{
   trace ();
   
   if (obj)
      obj->pos = pos;
   
   return obj;
}

/**
 * \brief Set the position on X of an object
 * \param obj the object to modify
 * \param x the position on X to set
 * \return \param obj
 */
Object* object_setPosX (Object *obj, int x)
{
   trace ();
   
   if (obj)
   {
      object_getPos (obj).x = x;
   }
   
   return obj;
}

/**
 * \brief Set the position on Y of an object
 * \param obj the object to modify
 * \param y the position on Y to set
 * \return \param obj
 */
Object* object_setPosY (Object *obj, int y)
{
   trace ();
   
   if (obj)
   {
      object_getPos (obj).y = y;
   }
   
   return obj;
}

/**
 * \brief Set the position of an object
 * \param obj the object to modify
 * \param x the position on X to set
 * \param y the position on Y to set
 * \return \param obj
 */
Object* object_setPos (Object *obj, int x, int y)
{
   trace ();
   
   if (obj)
   {
      object_getPos (obj).x = x;
      object_getPos (obj).y = y;
   }
   
   return obj;
}

/**
 * \brief Set the properties of an object
 * \param obj the object to modify
 * \param surface the surfact to put in the object
 * \param x the position on X of the object
 * \param y the position on Y of the object
 * \return \param obj
 */
Object* object_setData (Object *obj, SDL_Surface *surface, int x, int y)
{
   trace ();
   
   object_setSurface (obj, surface);
   object_setPos (obj, x, y);
   
   return obj;
}

/**
 * \brief Set the porperties of an object
 * \param obj the object to modify
 * \param surface the surfact to put in the object
 * \param pos the SDL_Rect to put in the object
 * \return \param obj
 */
Object* object_setDataFromSDL (Object *obj, SDL_Surface *surface, SDL_Rect pos)
{
   trace ();
   
   object_setSurface (obj, surface);
   object_setPosFromRect (obj, pos);
   
   return obj;
}



/** @} */
